import "reflect-metadata";
declare type Class = new (...arg: any[]) => any;
export declare type DecoratorTargetType = "Method" | "Class" | "Parameter" | "Property" | "Constructor";
export interface Decorator {
    targetType: DecoratorTargetType;
    target: string;
    value: any;
}
export interface ParameterDecorator extends Decorator {
    targetType: "Parameter";
    targetIndex: number;
}
export declare type Reflection = ParameterReflection | FunctionReflection | PropertyReflection | MethodReflection | ClassReflection | ObjectReflection;
export interface ReflectionBase {
    kind: string;
    name: string;
}
export interface ParameterReflection extends ReflectionBase {
    kind: "Parameter";
    decorators: any[];
    type?: any;
    typeClassification?: "Class" | "Array" | "Primitive";
}
export interface PropertyReflection extends ReflectionBase {
    kind: "Property";
    decorators: any[];
    type?: any;
    get: any;
    set: any;
    typeClassification?: "Class" | "Array" | "Primitive";
}
export interface MethodReflection extends ReflectionBase {
    kind: "Method";
    parameters: ParameterReflection[];
    returnType: any;
    decorators: any[];
    typeClassification?: "Class" | "Array" | "Primitive";
}
export interface ConstructorReflection extends ReflectionBase {
    kind: "Constructor";
    parameters: ParameterReflection[];
}
export interface FunctionReflection extends ReflectionBase {
    kind: "Function";
    parameters: ParameterReflection[];
    returnType: any;
}
export interface ClassReflection extends ReflectionBase {
    kind: "Class";
    ctor: ConstructorReflection;
    methods: MethodReflection[];
    properties: PropertyReflection[];
    decorators: any[];
    type: Class;
    typeClassification?: "Class" | "Array" | "Primitive";
}
export interface ObjectReflection extends ReflectionBase {
    kind: "Object";
    members: Reflection[];
}
export interface ArrayDecorator {
    kind: "Array";
    type: Class;
}
export interface TypeDecorator {
    kind: "Override";
    type: Class;
    info?: string;
}
export interface PrivateDecorator {
    kind: "Ignore";
}
export declare const DECORATOR_KEY = "plumier.key:DECORATOR";
export declare const DESIGN_TYPE = "design:type";
export declare const DESIGN_PARAMETER_TYPE = "design:paramtypes";
export declare const DESIGN_RETURN_TYPE = "design:returntype";
export declare function getParameterNames(fn: Function): string[];
export declare function getConstructorParameters(fn: Class): string[];
export declare function getMembers(fun: Function): string[];
export declare function useCache<K, P extends any[], R>(cache: Map<K, R>, fn: (...args: P) => R, getKey: (...args: P) => K): (...args: P) => R;
export declare function decorateParameter(callback: ((target: Class, name: string, index: number) => object)): (target: any, name: string, index: number) => void;
export declare function decorateParameter(data: {}): (target: any, name: string, index: number) => void;
export declare function decorateMethod(callback: ((target: Class, name: string) => object)): (target: any, name: string) => void;
export declare function decorateMethod(data: {}): (target: any, name: string) => void;
export declare function decorateProperty(callback: ((target: Class, name: string, index?: any) => object)): (target: any, name: string, index?: any) => void;
export declare function decorateProperty(data: {}): (target: any, name: string, index?: any) => void;
export declare function decorateClass(callback: ((target: Class) => object)): (target: any) => void;
export declare function decorateClass(data: {}): (target: any) => void;
export declare function decorate(data: any, targetTypes?: DecoratorTargetType[]): (...args: any[]) => void;
export declare function mergeDecorator(...fn: Function[]): (...args: any[]) => void;
/**
 * Reflect module
 * @param path module name
 */
export declare function reflect(path: string): ObjectReflection;
export declare namespace reflect {
    var ignore: () => (...args: any[]) => void;
    var type: (type: Class, info?: string | undefined) => (...args: any[]) => void;
    var array: (type: Class) => (...args: any[]) => void;
    var parameterProperties: () => (target: any) => void;
}
/**
 * Reflect class
 * @param classType Class
 */
export declare function reflect(classType: Class): ClassReflection;
export declare namespace reflect {
    var ignore: () => (...args: any[]) => void;
    var type: (type: Class, info?: string | undefined) => (...args: any[]) => void;
    var array: (type: Class) => (...args: any[]) => void;
    var parameterProperties: () => (target: any) => void;
}
export default reflect;
